@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 """
 Custom handlers may be created to handle other objects. Each custom handler
 must derive from :class:`jsonpickle.handlers.BaseHandler` and
@@ -21,7 +20,7 @@ objects that implement the reduce protocol::
     handlers.register(MyCustomObject, handlers.SimpleReduceHandler)
 
 """
-
+from __future__ import absolute_import, division, unicode_literals
 import collections
 import copy
 import datetime
@@ -29,10 +28,11 @@ import decimal
 import re
 import sys
 import time
+import uuid
 
-from jsonpickle import util
-from jsonpickle.compat import unicode
-from jsonpickle.compat import queue
+from . import util
+from .compat import queue
+from .compat import unicode
 
 
 class Registry(object):
@@ -63,11 +63,12 @@ class Registry(object):
         :param handler: The custom handler class (if None, a decorator wrapper is returned)
         :param base: Indicates whether the handler should be registered for all subclasses
 
-        This function can be also used as a decorator by omitting the `handler` argument:
+        This function can be also used as a decorator by omitting the `handler` argument::
+
+            @jsonpickle.handlers.register(Foo, base=True)
+            class FooHandler(jsonpickle.handlers.BaseHandler):
+                pass
 
-        @jsonpickle.handlers.register(Foo, base=True)
-        class FooHandler(jsonpickle.handlers.BaseHandler):
-            pass
         """
         if handler is None:
             def _register(handler_cls):
@@ -107,6 +108,15 @@ class BaseHandler(object):
         """
         self.context = context
 
+    def __call__(self, context):
+        """This permits registering either Handler instances or classes
+
+        :Parameters:
+          - `context`: reference to pickler/unpickler
+        """
+        self.context = context
+        return self
+
     def flatten(self, obj, data):
         """
         Flatten `obj` into a json-friendly form and write result to `data`.
@@ -187,69 +197,10 @@ class RegexHandler(BaseHandler):
     def restore(self, data):
         return re.compile(data['pattern'])
 
+
 RegexHandler.handles(type(re.compile('')))
 
 
-class SimpleReduceHandler(BaseHandler):
-    """Follow the __reduce__ protocol to pickle an object.
-
-    As long as the factory and its arguments are pickleable, this should
-    pickle any object that implements the reduce protocol.
-
-    """
-    def flatten(self, obj, data):
-        flatten = self.context.flatten
-        data['__reduce__'] = [flatten(i, reset=False) for i in obj.__reduce__()]
-        return data
-
-    def restore(self, data):
-        restore = self.context.restore
-        factory, args = [restore(i, reset=False) for i in data['__reduce__']]
-        return factory(*args)
-
-
-class OrderedDictReduceHandler(SimpleReduceHandler):
-    """Serialize OrderedDict on Python 3.4+
-
-    Python 3.4+ returns multiple entries in an OrderedDict's
-    reduced form.  Previous versions return a two-item tuple.
-    OrderedDictReduceHandler makes the formats compatible.
-
-    """
-    def flatten(self, obj, data):
-        # __reduce__() on older pythons returned a list of
-        # [key, value] list pairs inside a tuple.
-        # Recreate that structure so that the file format
-        # is consistent between python versions.
-        flatten = self.context.flatten
-        reduced = obj.__reduce__()
-        factory = flatten(reduced[0], reset=False)
-        pairs = [list(x) for x in reduced[-1]]
-        args = flatten((pairs,), reset=False)
-        data['__reduce__'] = [factory, args]
-        return data
-
-
-SimpleReduceHandler.handles(time.struct_time)
-SimpleReduceHandler.handles(datetime.timedelta)
-SimpleReduceHandler.handles(collections.deque)
-if sys.version_info >= (2, 7):
-    SimpleReduceHandler.handles(collections.Counter)
-    if sys.version_info >= (3, 4):
-        OrderedDictReduceHandler.handles(collections.OrderedDict)
-    else:
-        SimpleReduceHandler.handles(collections.OrderedDict)
-
-if sys.version_info >= (3, 0):
-    SimpleReduceHandler.handles(decimal.Decimal)
-
-try:
-    import posix
-    SimpleReduceHandler.handles(posix.stat_result)
-except ImportError:
-    pass
-
-
 class QueueHandler(BaseHandler):
     """Opaquely serializes Queue objects
 
@@ -278,3 +229,16 @@ class CloneFactory(object):
 
     def __repr__(self):
         return ('<CloneFactory object at 0x%x (%s)>' % (id(self), self.exemplar))
+
+
+class UUIDHandler(BaseHandler):
+    """Serialize uuid.UUID objects"""
+
+    def flatten(self, obj, data):
+        data['hex'] = obj.hex
+        return data
+
+    def restore(self, data):
+        return uuid.UUID(data['hex'])
+
+UUIDHandler.handles(uuid.UUID)
