@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2008 John Paulett (john -at- paulett.org)
-# Copyright (C) 2009, 2011, 2013 David Aguilar (davvid -at- gmail.com)
+# Copyright (C) 2009, 2011, 2013 David Aguilar (davvid -at- gmail.com) and contributors
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
-
-import sys
+from __future__ import absolute_import, division, unicode_literals
+import base64
 import quopri
+import sys
 
-import jsonpickle.util as util
-import jsonpickle.tags as tags
-import jsonpickle.handlers as handlers
-
-from jsonpickle.compat import set
-from jsonpickle.backend import JSONBackend
+from . import util
+from . import tags
+from . import handlers
+from .compat import numeric_types, set, unicode
+from .backend import JSONBackend
 
 
 def decode(string, backend=None, context=None, keys=False, reset=True,
@@ -64,38 +64,48 @@ class _Proxy(object):
     def __init__(self):
         self.instance = None
 
+    def get(self):
+        return self.instance
+
+    def reset(self, instance):
+        self.instance = instance
+
+class _IDProxy(_Proxy):
+    def __init__(self, objs, index):
+        self._index = index
+        self._objs = objs
+
+    def get(self):
+        return self._objs[self._index]
+
 
 def _obj_setattr(obj, attr, proxy):
-    setattr(obj, attr, proxy.instance)
+    setattr(obj, attr, proxy.get())
 
 
 def _obj_setvalue(obj, idx, proxy):
-    obj[idx] = proxy.instance
+    obj[idx] = proxy.get()
 
 
 class Unpickler(object):
 
     def __init__(self, backend=None, keys=False, safe=False):
-        # The current recursion depth
-        # Maps reference names to object instances
         self.backend = _make_backend(backend)
         self.keys = keys
         self.safe = safe
 
-        self._namedict = {}
-        # The namestack grows whenever we recurse into a child object
-        self._namestack = []
-
-        # Maps objects to their index in the _objs list
-        self._obj_to_idx = {}
-        self._objs = []
-        self._proxies = []
+        self.reset()
 
     def reset(self):
         """Resets the object's internal state.
         """
+        # Map reference names to object instances
         self._namedict = {}
+
+        # The stack of names traversed for child objects
         self._namestack = []
+
+        # Map of objects to their index in the _objs list
         self._obj_to_idx = {}
         self._objs = []
         self._proxies = []
@@ -106,10 +116,10 @@ class Unpickler(object):
         Simply returns any of the basic builtin types
 
         >>> u = Unpickler()
-        >>> u.restore('hello world')
-        'hello world'
-        >>> u.restore({'key': 'value'})
-        {'key': 'value'}
+        >>> u.restore('hello world') == 'hello world'
+        True
+        >>> u.restore({'key': 'value'}) == {'key': 'value'}
+        True
 
         """
         if reset:
@@ -123,10 +133,13 @@ class Unpickler(object):
         """Replace proxies with their corresponding instances"""
         for (obj, attr, proxy, method) in self._proxies:
             method(obj, attr, proxy)
+        self._proxies = []
 
     def _restore(self, obj):
-        if has_tag(obj, tags.BYTES):
-            restore = self._restore_bytestring
+        if has_tag(obj, tags.B64):
+            restore = self._restore_base64
+        elif has_tag(obj, tags.BYTES):  # Backwards compatibility
+            restore = self._restore_quopri
         elif has_tag(obj, tags.ID):
             restore = self._restore_id
         elif has_tag(obj, tags.REF):  # Backwards compatibility
@@ -155,7 +168,11 @@ class Unpickler(object):
             restore = lambda x: x
         return restore(obj)
 
-    def _restore_bytestring(self, obj):
+    def _restore_base64(self, obj):
+        return base64.decodestring(obj[tags.B64].encode('utf-8'))
+
+    #: For backwards compatibility with bytes data produced by older versions
+    def _restore_quopri(self, obj):
         return quopri.decodestring(obj[tags.BYTES].encode('utf-8'))
 
     def _restore_iterator(self, obj):
@@ -167,12 +184,19 @@ class Unpickler(object):
         Assumes that iterator items (the last two) are represented as lists
         as per pickler implementation.
         """
+        proxy = _Proxy()
+        self._mkref(proxy)
         reduce_val = obj[tags.REDUCE]
-        f, args, state, listitems, dictitems = map(self._restore, reduce_val)
+        reduce_tuple = f, args, state, listitems, dictitems = map(self._restore, reduce_val)
+
         if f == tags.NEWOBJ or f.__name__ == '__newobj__':
             # mandated special case
             cls = args[0]
+            if not isinstance(cls, type):
+                cls = self._restore(cls)
+
             stage1 = cls.__new__(cls, *args[1:])
+
         else:
             stage1 = f(*args)
 
@@ -182,11 +206,25 @@ class Unpickler(object):
             except AttributeError:
                 # it's fine - we'll try the prescribed default methods
                 try:
-                    stage1.__dict__.update(state)
+                    # we can't do a straight update here because we
+                    # need object identity of the state dict to be
+                    # preserved so that _swap_proxies works out
+                    for k in stage1.__dict__.keys():
+                        if k not in state:
+                            state[k] = stage1.__dict__[k]
+                    stage1.__dict__ = state
                 except AttributeError:
                     # next prescribed default
-                    for k, v in state.items():
-                        setattr(stage1, k, v)
+                    try:
+                        for k, v in state.items():
+                            setattr(stage1, k, v)
+                    except:
+                        dict_state, slots_state = state
+                        if dict_state:
+                            stage1.__dict__.update(dict_state)
+                        if slots_state:
+                            for k, v in slots_state.items():
+                                setattr(stage1, k, v)
 
         if listitems:
             # should be lists if not None
@@ -200,10 +238,16 @@ class Unpickler(object):
             for k, v in dictitems:
                 stage1.__setitem__(k, v)
 
+        proxy.reset(stage1)
+        self._swapref(proxy, stage1)
         return stage1
 
     def _restore_id(self, obj):
-        return self._objs[obj[tags.ID]]
+        try:
+            idx = obj[tags.ID]
+            return self._objs[idx]
+        except IndexError:
+            return _IDProxy(self._objs, idx)
 
     def _restore_ref(self, obj):
         return self._namedict.get(obj[tags.REF])
@@ -226,8 +270,12 @@ class Unpickler(object):
         cls = loadclass(class_name)
         handler = handlers.get(cls, handlers.get(class_name))
         if handler is not None:  # custom handler
+            proxy = _Proxy()
+            self._mkref(proxy)
             instance = handler(self).restore(obj)
-            return self._mkref(instance)
+            proxy.reset(instance)
+            self._swapref(proxy, instance)
+            return instance
 
         if cls is None:
             return self._mkref(obj)
@@ -287,15 +335,15 @@ class Unpickler(object):
                 except:  # fail gracefully
                     return self._mkref(obj)
 
-        proxy.instance = instance
+        proxy.reset(instance)
         self._swapref(proxy, instance)
 
         if isinstance(instance, tuple):
             return instance
 
         if (hasattr(instance, 'default_factory') and
-                type(instance.default_factory) is _Proxy):
-            instance.default_factory = instance.default_factory.instance
+                isinstance(instance.default_factory, _Proxy)):
+            instance.default_factory = instance.default_factory.get()
 
         return self._restore_object_instance_variables(obj, instance)
 
@@ -307,7 +355,11 @@ class Unpickler(object):
             # ignore the reserved attribute
             if ignorereserved and k in tags.RESERVED:
                 continue
-            self._namestack.append(k)
+            if isinstance(k, numeric_types):
+                str_k = unicode(k)
+            else:
+                str_k = k
+            self._namestack.append(str_k)
             k = restore_key(k)
             # step into the namespace
             value = self._restore(v)
@@ -319,7 +371,7 @@ class Unpickler(object):
 
             # This instance has an instance variable named `k` that is
             # currently a proxy and must be replaced
-            if type(value) is _Proxy:
+            if isinstance(value, _Proxy):
                 self._proxies.append((instance, k, value, method))
 
             # step out
@@ -375,8 +427,8 @@ class Unpickler(object):
         parent.extend(children)
         method = _obj_setvalue
         proxies = [(parent, idx, value, method)
-                   for idx, value in enumerate(parent)
-                   if isinstance(value, _Proxy)]
+                    for idx, value in enumerate(parent)
+                        if isinstance(value, _Proxy)]
         self._proxies.extend(proxies)
         return parent
 
@@ -390,9 +442,17 @@ class Unpickler(object):
         data = {}
         restore_key = self._restore_key_fn()
         for k, v in sorted(obj.items(), key=util.itemgetter):
-            self._namestack.append(k)
+            if isinstance(k, numeric_types):
+                str_k = unicode(k)
+            else:
+                str_k = k
+            self._namestack.append(str_k)
             k = restore_key(k)
             data[k] = self._restore(v)
+            # k is currently a proxy and must be replaced
+            if isinstance(data[k], _Proxy):
+                self._proxies.append((data, k, data[k], _obj_setvalue))
+
             self._namestack.pop()
         return data
 
@@ -433,16 +493,14 @@ class Unpickler(object):
 
         >>> u = Unpickler()
         >>> u._namestack = []
-        >>> u._refname()
-        '/'
-
+        >>> u._refname() == '/'
+        True
         >>> u._namestack = ['a']
-        >>> u._refname()
-        '/a'
-
+        >>> u._refname() == '/a'
+        True
         >>> u._namestack = ['a', 'b']
-        >>> u._refname()
-        '/a/b'
+        >>> u._refname() == '/a/b'
+        True
 
         """
         return '/' + '/'.join(self._namestack)
@@ -463,10 +521,11 @@ class Unpickler(object):
         proxy_id = id(proxy)
         instance_id = id(instance)
 
-        self._obj_to_idx[instance_id] = self._obj_to_idx[proxy_id]
+        instance_index = self._obj_to_idx[proxy_id]
+        self._obj_to_idx[instance_id] = instance_index
         del self._obj_to_idx[proxy_id]
 
-        self._objs[-1] = instance
+        self._objs[instance_index] = instance
         self._namedict[self._refname()] = instance
 
 
