@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import base64
 import collections
 import decimal
 import re
@@ -11,6 +12,7 @@ import jsonpickle
 from jsonpickle import handlers
 from jsonpickle import tags
 from jsonpickle.compat import queue, set, unicode, bytes, PY2, PY_MINOR
+import jsonpickle.util as util
 
 from helper import SkippableTest
 
@@ -427,11 +429,11 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(newobj.c.name, 'c')
 
     def test_newstyleslots_iterable(self):
-        obj = ThingWithIterableSlots('a', 'b')
+        obj = ThingWithIterableSlots('alpha', 'bravo')
         jsonstr = jsonpickle.encode(obj)
         newobj = jsonpickle.decode(jsonstr)
-        self.assertEqual(newobj.a, 'a')
-        self.assertEqual(newobj.b, 'b')
+        self.assertEqual(newobj.a, 'alpha')
+        self.assertEqual(newobj.b, 'bravo')
 
     def test_newstyleslots_string_slot(self):
         obj = ThingWithStringSlots('a', 'b')
@@ -519,6 +521,22 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertTrue('<BrokenReprThing "test">' in flattened)
         self.assertTrue(flattened['<BrokenReprThing "test">'])
 
+
+    def test_ordered_dict_reduces(self):
+        if sys.version_info < (2, 7):
+            return
+
+        d = collections.OrderedDict()
+        d.update(c=3)
+        d.update(a=1)
+        d.update(b=2)
+
+        has_reduce, has_reduce_ex = util.has_reduce(d)
+
+        self.assertTrue(util.is_reducible(d))
+        self.assertTrue(has_reduce or has_reduce_ex)
+
+        
     def test_ordered_dict(self):
         if sys.version_info < (2, 7):
             return
@@ -669,7 +687,6 @@ class AdvancedObjectsTestCase(SkippableTest):
         flat = self.pickler.flatten(obj)
         actual = flat[tags.STATE]
         self.assertEqual(expect, actual)
-
         restored = self.unpickler.restore(flat)
         self.assertEqual(expect, restored)
 
@@ -759,12 +776,13 @@ class AdvancedObjectsTestCase(SkippableTest):
         # bytestrings are wrapped in PY3 but in PY2 we try to decode first
         encoded = self.pickler.flatten(b1)
         if PY2:
-            self.assertTrue(encoded == u1)
-            self.assertTrue(type(encoded) is unicode)
+            self.assertEqual(encoded, u1)
+            self.assertEqual(type(encoded), unicode)
         else:
-            self.assertTrue(encoded != u1)
-            self.assertTrue(encoded == {tags.BYTES: 'foo'})
-            self.assertTrue(type(encoded[tags.BYTES]) is unicode)
+            self.assertNotEqual(encoded, u1)
+            b64ustr= base64.encodestring(b'foo').decode('utf-8')
+            self.assertEqual({tags.B64: b64ustr}, encoded)
+            self.assertEqual(type(encoded[tags.B64]), unicode)
         decoded = self.unpickler.restore(encoded)
         self.assertTrue(decoded == b1)
         if PY2:
@@ -774,13 +792,29 @@ class AdvancedObjectsTestCase(SkippableTest):
 
         # bytestrings that we can't decode to UTF-8 will always be wrapped
         encoded = self.pickler.flatten(b2)
-        self.assertTrue(encoded != b2)
-        self.assertTrue(encoded == {tags.BYTES: 'foo=FF'})
-        self.assertTrue(type(encoded[tags.BYTES]) is unicode)
+        self.assertNotEqual(encoded, b2)
+        b64ustr= base64.encodestring(b'foo\xff').decode('utf-8')
+        self.assertEqual({tags.B64: b64ustr}, encoded)
+        self.assertEqual(type(encoded[tags.B64]), unicode)
         decoded = self.unpickler.restore(encoded)
-        self.assertTrue(decoded == b2)
+        self.assertEqual(decoded, b2)
         self.assertTrue(type(decoded) is bytes)
 
+    def test_backcompat_bytes_quoted_printable(self):
+        """Test decoding bytes objects from older jsonpickle versions"""
+
+        b1 = b'foo'
+        b2 = b'foo\xff'
+
+        # older versions of jsonpickle used a quoted-printable encoding
+        expect = b1
+        actual = self.unpickler.restore({tags.BYTES: unicode('foo')})
+        self.assertEqual(expect, actual)
+
+        expect = b2
+        actual = self.unpickler.restore({tags.BYTES: unicode('foo=FF')})
+        self.assertEqual(expect, actual)
+
     def test_nested_objects(self):
         obj = ThingWithTimedeltaAttribute(99)
         flattened = self.pickler.flatten(obj)
